# Copyright (C) 2023 p-ban.com Corp.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

import sys
from decimal import *
import urllib.parse

import webbrowser
import wx
import pcbnew

from pban import *


def getBoardSize(board):
    drawings = [d for d in board.GetDrawings() if d.GetLayer() == pcbnew.Edge_Cuts]
    if len(drawings) < 1:
        return 0, 0
    line_bak = [drawing.GetWidth() for drawing in drawings]
    for drawing in drawings:
        drawing.SetWidth(pcbnew.FromMM(0.001))
    box = board.GetBoardEdgesBoundingBox()
    w = box.GetWidth()
    h = box.GetHeight()
    for l, d in zip(line_bak, drawings):
        d.SetWidth(l)
    return round_half_up(pcbnew.ToMM(w)), round_half_up(pcbnew.ToMM(h))


def getMinLineThicknessMM(board):
    track_width = [t.GetWidth() for t in board.GetTracks() if t.Type() == pcbnew.PCB_TRACE_T]
    if not len(track_width):
        return 0.127
    return pcbnew.ToMM(min(track_width))


def getMinViaDiaDrillMM(board):
    vias = [(v.GetDrillValue(), v.GetWidth()) for v in board.GetTracks() if v.Type() == pcbnew.PCB_VIA_T]
    if not len(vias):
        return (0.3, 0.6)
    vias = sorted(vias)
    return vias[0]


def clickOpen(url):
    def onEvent(e):
        if e.LeftUp():
            webbrowser.open(url)
        e.Skip()
    return onEvent


def showWarning(msg, url):
    x = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_X) / 2
    y = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_Y) / 2
    dialog = wx.Dialog(None, id=wx.ID_ANY, title="Warning", pos=(x, y), style=wx.CLOSE_BOX | wx.STAY_ON_TOP)
    sizer = wx.BoxSizer(wx.VERTICAL)
    # message
    msg = wx.StaticText(dialog, wx.ID_ANY, msg)
    mFont = msg.GetFont()
    # url
    if url:
        link = wx.StaticText(dialog, wx.ID_ANY, url)
        lFont = link.GetFont()
        lFont.Bold()
        lFont.SetUnderlined(True)
        link.SetFont(lFont)
        ev_func = clickOpen(url)
        link.Bind(wx.EVT_MOUSE_EVENTS, ev_func)
        link.SetForegroundColour('#006699')
    # button
    btn = wx.Button(dialog, wx.ID_OK, label="OK")
    # Layout
    sizer.Add(msg, 0, wx.ALL, 10)
    if url:
        sizer.Add(link, 0, wx.ALIGN_CENTER, 0)
    sizer.Add(btn, 0, wx.ALIGN_CENTER, 0)
    sizer.SetSizeHints(dialog)
    dialog.SetSizer(sizer)
    try:
        dialog.ShowModal()
    finally:
        dialog.Destroy()


class PBanQuotation:
    PBAN_URL = "https://www.p-ban.com/kiban/com/csComEst.do"

    def __init__(self, board=None):
        self.params = {
            "serviceM": "true",
            "serviceG": "false",
            "viaSite": "3",
        }
        if not board:
            board = pcbnew.GetBoard()

        try:
            self.prepare_params(board)
            self.encoded = urllib.parse.urlencode(self.params)
            self.is_valid = True
        except NotSupportedParams as e:
            self.is_valid = False

    def prepare_params(self, board):
        width, height = getBoardSize(board)
        if width == 0 or height == 0:
            raise NotSupportedParams()
        self.params["sizeX"] = width
        self.params["sizeY"] = height
        layer_count = board.GetCopperLayerCount()
        try:
            layer = Layer.choice(layer_count)
        except NotSupportedParams as e:
            raise e
        self.params["layer"] = layer.value
        # ds = board.GetDesignSettings()
        # brd_thickness = pcbnew.ToMM(ds.GetBoardThickness())
        # try:
        #     bt = BoardThickness.choice(brd_thickness, layer_count)
        #     self.params["boardThickness"] = bt.value
        # except NotSupportedParams as e:
        #     raise e

        # lineThickness = getMinLineThicknessMM(board)
        # try:
        #     pattern = MinPattern.pattern(lineThickness)
        #     if not pattern.validate(KibanType.RIGID):
        #         raise NotSupportedParams()
        #     self.params["minPattern"] = pattern.value
        # except NotSupportedParams as e:
        #     raise e
        # hole_min, via_min = getMinViaDiaDrillMM(board)
        # via_min = pcbnew.ToMM(via_min)
        # hole_min = pcbnew.ToMM(hole_min)
        # try:
        #     self.params["minLand"] = MinLand.choice(hole_min, via_min).value
        # except NotSupportedParams as e:
        #     raise e
        self.params["cnt"] = 1


    def open(self):
        if self.is_valid:
            showWarning(W_MSG_OPEN, None)
            webbrowser.open(self.PBAN_URL + "?" + self.encoded)
        else:
            if "sizeX" not in self.params or "sizeY" not in self.params:
                showWarning(W_MSG_SIZES, None)
            elif "layer" not in self.params:
                showWarning(W_MSG_LAYERS, W_URL_LAYERS)
            # elif "boardThickness" not in self.params:
            #     showWarning(W_MSG_BOARDTHICKNESS, W_URL_BOARDTHICKNESS)
            # elif "minPattern" not in self.params:
            #     showWarning(W_MSG_PATTERNS, W_URL_PATTERNS)
            # elif "minLand" not in self.params:
            #     showWarning(W_MSG_LAND, W_URL_LAND)


def round_half_up(d):
    if type(d) != Decimal:
        d = Decimal(d)
    return d.quantize(Decimal("0.1"))
